#!/bin/bash

_usage(){
	echo "Usage: sh $0 <database> <dbuser> <dbpassword> [dbport] [dbhost]\
		\n"
	echo "$*"
} 

_error(){
	echo "ERROR: $*" >&2
	kill $$
	exit -1
}       

_bye(){
	echo "BYE: $*"
	exit 0 
}       

rmkdir(){
	up=`echo $1 | sed 's/\/[^\/]*$//'`
	if ! [ -d $up ]; then rmkdir $up; fi
	mkdir -m 755 $1
}

trim(){
	echo $*
}       


if [ $# -lt 3 ]; then _usage; exit 0; fi 

db=$1;
if [ "X$1" = "X" ]; then _usage; exit 0; fi	 

dbuser=$2
if [ "X$2" = "X" ]; then _usage; fi	 

dbpassword=$3
if [ "X$3" = "X" ]; then _usage; fi	 

if [ $4 ]; then	dbport=$4; fi
if [ "X$4" = "X" ]; then dbport=5432; fi	 

if [ $5 ]; then	dbhost=$5; fi
if [ "X$5" = "X" ]; then dbhost=localhost; fi	 


if test "$EUID" -ne 0 ; then
     _error 'Please run as root'
fi

wd=$(cd "$(dirname "$0")"; pwd) 

[ -f "$wd/.path.extension" ] && ct=`cat $wd/.path.extension` || \
ct=`dpkg -l |grep postgres |grep contrib |awk '{print "dpkg -L "$2}' |sh |egrep 'postgresql.*/extension$'`
[ -d "${ct}" ] || _error "Unable to find postgresql extension dir: $ct"

[ -f "$wd/.path.lib" ] && ctl=`cat $wd/.path.lib` || \
ctl=`dpkg -l |grep postgres |grep contrib |awk '{print "dpkg -L "$2}' |sh |egrep 'postgresql.*/lib$'`
[ -d "${ctl}" ] || _error "Unable to find postgresql lib dir: $ctl"

dt=`date +"%Y%m%d%H%M"`

[ -f "$wd/.suid" ] && tmpu=`cat $wd/.suid` || tmpu=postgres


tmpd=`sudo -u $tmpu mktemp -d -t __vitml.db.2.XXXXXX`
chmod 777 $tmpd

mysession=`echo "${tmpd}" | sed 's/.*\.\(.*\)$/\1/'`

_mymajor=`cat $wd/.version.major`
_myminor=`cat $wd/.version.minor`
_myrelease=`cat $wd/.version.release`
myrevision=`cat $wd/.revision`
mybuild=`cat $wd/.build`
myrc=`cat $wd/.rc`
mypassport=`cat $wd/.passport`
myos=linux
uname -a | grep -q _64 && mymachine=x64 || mymachine=x86

export PGPORT=$dbport
export PGPASSWORD=postgres

datadir=`psql -h $dbhost -p $dbport -q -t -c "select current_setting('data_directory')" postgres postgres |awk '{print $1}'`
contrib=`cat $wd/.contrib |tr "\n\r" "  "`
config=`cat $wd/.configure |tr -d "\n\r"`

cat /dev/null > $tmpd/vars.psql

sed 's#%%VERSION_MAJOR%%#'${_mymajor}'#g;s#%%VERSION_MINOR%%#'${_myminor}'#g;s#%%VERSION_RELEASE%%#'${_myrelease}'#g;s#%%BUILD%%#'${mybuild}'#g;s#%%REVISION%%#'${myrevision}'#g;s#%%RC%%#'${myrc}'#g;s#%%PASSPORT%%#'${mypassport}'#g;s#%%DATADIR%%#'${datadir}'#g;s#%%DBNAME%%#'${db}'#g;s#%%DBUSER%%#'${dbuser}'#g;s#%%DBPASS%%#'${dbpassword}'#g;s#%%CONTRIBDIR%%#'${ct}'#g;s#%%CONTRIBLIBDIR%%#'${ctl}'#g;s#%%OS%%#'${myos}'#g;s#%%MACHINE%%#'${mymachine}'#g;s#%%CONTRIBLIST%%#'"${contrib}"'#g;' $wd/vars.psql_ > $tmpd/vars.psql

cat /dev/null > $tmpd/_res

psql -h $dbhost -p $dbport -q -t -v _DIR="${wd}" -v _TMP="${tmpd}" postgres  postgres < $wd/install.sql 2>&1 |grep ERROR\\\|FATAL > $tmpd/_res
res=`cat $tmpd/_res`
if [ "X${res}" != "X" ]; then
#	dropdb ${db} --if-exists -U postgres -h localhost -p $dbport
	cat $tmpd/_res
	_error "[FATAL] Error setting DB up "
fi

echo "...edge config:"

export PGPASSWORD=$dbpassword
cat <<EOF | psql -h $dbhost -p $dbport -w $db $dbuser | grep -q ERROR && _error "failed to init autocode database!"
begin;
select core.db_init();
commit;
EOF


rm -rf $tmpd

echo "ALL DONE!"
