
create or replace function binfs._meta_get( ipath text ) returns text as $$
    import os
    try:
        from bsddb import db
    except ImportError:
        from bsddb3 import db
    if not ipath: return None
    _r = None
    _t = ipath
    [_t,key] = os.path.split(_t)
    [dir,block] = os.path.split(_t)
    _db = db.DB()
    try:
        _db.open(os.path.join(dir,".%s..meta"%(block)), dbtype=db.DB_BTREE, flags=db.DB_RDONLY)
        _r = _db.get( key )
    except db.DBNotFoundError:
        return None
    except Exception as ex:    
        plpy.execute("select core.log('binfs.meta.get', 'error', %s)" % ( plpy.quote_nullable(str(ex))) )
        return None
    return _r 
$$ language plpythonu;


create or replace function binfs._meta_set( ipath text, idata text, iover boolean default true, ipagesize integer default 512 ) returns integer as $$
    import os
    try:
        from bsddb import db
    except ImportError:
        from bsddb3 import db
    if not ipath: return None    
    _r = None
    _t = ipath
    [_t,key] = os.path.split(_t)
    [dir,block] = os.path.split(_t)
    _db = db.DB()
    _f = os.path.join(dir,".%s..meta"%(block))
    if ipagesize: _db.set_pagesize(ipagesize)
    try:
        _db.open(os.path.join(dir,".%s..meta"%(block)), dbtype=db.DB_BTREE, flags=db.DB_CREATE)
        _db.put(key, idata, flags = db.DB_NOOVERWRITE if not iover else 0 )
        _r = len(idata)
    except Exception as ex:    
        plpy.execute("select core.log('binfs.meta.set', 'error', %s)" % ( plpy.quote_nullable(str(ex))) )
        return None
    return _r
$$ language plpythonu;


create or replace function binfs._meta_drop( ipath text ) returns integer as $$
    import os
    try:
        from bsddb import db
    except ImportError:
        from bsddb3 import db
    if not ipath: return None
    _r = None
    _t = ipath
    [_t,key] = os.path.split(_t)
    [dir,block] = os.path.split(_t)
    _db = db.DB()
    try:
        _db.open(os.path.join(dir,".%s..meta"%(block)), dbtype=db.DB_BTREE )
        _db.delete( key )
        _r = 1
    except db.DBNotFoundError:
        _r = 0
    except Exception as ex:    
        plpy.execute("select core.log('binfs.meta.drop', 'error', %s)" % ( plpy.quote_nullable(str(ex))) )
        return None
    return _r 
$$ language plpythonu;



create or replace function binfs.meta_get( istore varchar, ikey anyelement ) returns text as $$
select binfs._meta_get( binfs.path(istore, ikey) );
$$ language sql;

create or replace function binfs.meta_set( istore varchar, ikey anyelement, idata text, iover boolean default true ) returns integer as $$
select binfs._meta_set( binfs.path(istore, ikey), idata, iover, _bo('meta_page_size')::integer );
$$ language sql;


create or replace function binfs.meta_drop( istore varchar, ikey anyelement ) returns integer as $$
select binfs._meta_drop( binfs.path(istore, ikey) );
$$ language sql;


